@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2019 Andreas Steffen
+ * Copyright (C) 2017-2022 Andreas Steffen
  * Copyright (C) 2010 Martin Willi
  *
  * Copyright (C) secunet Security Networks AG
@@ -27,22 +27,13 @@
 #include <asn1/asn1.h>
 
 
-/**
- * Entry for a revoked certificate
- */
-typedef struct {
-	chunk_t serial;
-	crl_reason_t reason;
-	time_t date;
-} revoked_t;
-
 /**
  * Add a revocation to the list
  */
 static void add_revoked(linked_list_t *list,
 						chunk_t serial, crl_reason_t reason, time_t date)
 {
-	revoked_t *revoked;
+	crl_revoked_t *revoked;
 
 	INIT(revoked,
 		.serial = chunk_clone(serial),
@@ -55,7 +46,7 @@ static void add_revoked(linked_list_t *list,
 /**
  * Destroy a reason entry
  */
-static void revoked_destroy(revoked_t *revoked)
+static void revoked_destroy(crl_revoked_t *revoked)
 {
 	free(revoked->serial.ptr);
 	free(revoked);
@@ -64,7 +55,7 @@ static void revoked_destroy(revoked_t *revoked)
 CALLBACK(filter, bool,
 	void *data, enumerator_t *orig, va_list args)
 {
-	revoked_t *revoked;
+	crl_revoked_t *revoked;
 	crl_reason_t *reason;
 	chunk_t *serial;
 	time_t *date;
